==> brain-py-2.0.1.tar.gz: brain-py-2.0.1/brainpy/integrators/ode/base.py <==
# -*- coding: utf-8 -*-


import abc

from brainpy import math
from brainpy.integrators import constants, utils
from brainpy.integrators.base import Integrator

__all__ = [
  'ODEIntegrator',
]


def f_names(f):
  func_name = constants.unique_name('ode')
  if f.__name__.isidentifier():
    func_name += '_' + f.__name__
  return func_name


class ODEIntegrator(Integrator):
  """Numerical Integrator for Ordinary Differential Equations (ODEs).
  """

  def __init__(self, f, var_type=None, dt=None, name=None, show_code=False):
    super(ODEIntegrator, self).__init__(name=name)

    # others
    self.dt = math.get_dt() if dt is None else dt
    assert isinstance(self.dt, (int, float)), f'"dt" must be a float, but got {self.dt}'
    self.show_code = show_code

    # derivative function
    self.derivative = {constants.F: f}
    self.f = f

    # integration function
    self.integral = None

    # parse function arguments
    variables, parameters, arguments = utils.get_args(f)
    self.variables = variables  # variable names, (before 't')
    self.parameters = parameters  # parameter names, (after 't')
    self.arguments = list(arguments) + [f'{constants.DT}={self.dt}']  # function arguments
    self.var_type = var_type  # variable type

    # code scope
    self.code_scope = {constants.F: f}

    # code lines
    self.func_name = f_names(f)
    self.code_lines = [f'def {self.func_name}({", ".join(self.arguments)}):']

  @abc.abstractmethod
  def build(self):
    raise NotImplementedError('Must implement how to build your step function.')

  def __call__(self, *args, **kwargs):
    assert self.integral is not None, 'Please build the integrator first.'
    return self.integral(*args, **kwargs)

==> brain-py-2.1.0.tar.gz: brain-py-2.1.0/brainpy/integrators/ode/base.py <==
# -*- coding: utf-8 -*-


import abc

from brainpy import math
from brainpy.integrators import constants, utils
from brainpy.integrators.base import Integrator

__all__ = [
  'ODEIntegrator',
]


def f_names(f):
  func_name = constants.unique_name('ode')
  if f.__name__.isidentifier():
    func_name += '_' + f.__name__
  return func_name


class ODEIntegrator(Integrator):
  """Numerical Integrator for Ordinary Differential Equations (ODEs).
  """

  def __init__(self, f, var_type=None, dt=None, name=None, show_code=False):

    dt = math.get_dt() if dt is None else dt
    parses = utils.get_args(f)
    variables = parses[0]  # variable names, (before 't')
    parameters = parses[1]  # parameter names, (after 't')
    arguments = parses[2]   # function arguments

    # super initialization
    super(ODEIntegrator, self).__init__(name=name,
                                        variables=variables,
                                        parameters=parameters,
                                        arguments=arguments,
                                        dt=dt)

    # others
    self.show_code = show_code
    self.var_type = var_type  # variable type

    # derivative function
    self.derivative = {constants.F: f}
    self.f = f

    # code scope
    self.code_scope = {constants.F: f}

    # code lines
    self.func_name = f_names(f)
    self.code_lines = [f'def {self.func_name}({", ".join(self.arguments)}):']

  @abc.abstractmethod
  def build(self):
    raise NotImplementedError('Must implement how to build your step function.')
